local _, LHP = ...;

local Config = {};

-- Load Fonts and set the sizes etc
local Font = "Interface\\Addons\\LHPToolkit\\media\\fonts\\impfont.ttf";
local CheckBoxFontSize = 13;
local CheckBoxOffset = -30;
local HeaderFontSize = 13;

-- Simply checks if any of the options have changed. This is basically a huge if statement
local function ConfigChanged()
    if(Conf_Bloodlust ~= Config.panel.bloodLust:GetChecked() or Conf_SurrenderSound ~= Config.panel.surrenderSound:GetChecked() or Conf_SurrenderText ~= Config.panel.surrenderText:GetChecked() or Conf_BennyHill ~= Config.panel.bennyHill:GetChecked()) then
        return true;
    else
        return false;
    end
end

--[[
    Primary Window Config Stuff Begins
]]
-- Reset all options to the default settings
local function SetDefaults_Primary()
    Config.panel.surrenderSound:SetChecked(true);
    Config.panel.surrenderText:SetChecked(true);
    Config.panel.bennyHill:SetChecked(true);
    Config.panel.bloodLust:SetChecked(true);
end

-- Loads the already set config options for the Primary window
local function LoadConfig_Primary()
    Config.panel.surrenderSound:SetChecked(Conf_SurrenderSound);
    Config.panel.surrenderText:SetChecked(Conf_SurrenderText);
    Config.panel.bennyHill:SetChecked(Conf_BennyHill);
    Config.panel.bloodLust:SetChecked(Conf_Bloodlust);
end

-- Applies any changes
local function ApplyChanges_Primary()
    if(ConfigChanged()) then
        Conf_SurrenderSound = Config.panel.surrenderSound:GetChecked();
        Conf_SurrenderText = Config.panel.surrenderText:GetChecked();
        Conf_BennyHill = Config.panel.bennyHill:GetChecked();
        Conf_Bloodlust = Config.panel.bloodLust:GetChecked();
        ReloadUI();
    end
end

-- Checks to see if any of the values are nil. If they are it must be a first load. Set to defaults.
local function CheckFirstLoad()
    if (Conf_SurrenderSound == nil) then Conf_SurrenderSound = true end
    if (Conf_SurrenderText == nil) then Conf_SurrenderText = true end
    if (Conf_BennyHill == nil) then Conf_BennyHill = true end
    if (Conf_Bloodlust == nil) then Conf_Bloodlust = true end
end

-- Event Handler, Only used for detecting when the addon has finished initialising and trigger config loading
local function HandleEvents(self, event, ...)
    if(event == "ADDON_LOADED" and ... == "LHPToolkit") then
        CheckFirstLoad();
        LoadConfig_Primary();
    end
end

-- Builds the Primary (Misc Settings) Config Panel - Parent to all other panels
local function BuildWindow_Primary()
    Config.panel = CreateFrame("Frame", "LHP_Misc", UIParent);
    Config.panel.name = "LHPToolkit";
    Config.panel.okay = ApplyChanges_Primary;
    Config.panel.cancel = LoadConfig_Primary;
    Config.panel.default = SetDefaults_Primary;

    -- Register the event handler and addon loaded event
    Config.panel:SetScript("OnEvent", HandleEvents);
    Config.panel:RegisterEvent("ADDON_LOADED");

    -- Title
    Config.panel.titleText = Config.panel:CreateFontString(nil, "OVERLAY", "GameFontNormal");
    Config.panel.titleText:SetFont(Font, 14, "OUTLINE");
    Config.panel.titleText:SetPoint("TOPLEFT", 190, -10);
    Config.panel.titleText:SetText("|cffffff00 Left Hand Path Toolkit - v"..GetAddOnMetadata("LHPToolkit", "Version"));

    --[[
    Misc Config Begins
    ]]
    -- Misc Header
    Config.panel.miscHeader = Config.panel:CreateFontString(nil, "OVERLAY", "GameFontNormal");
    Config.panel.miscHeader:SetFont(Font, HeaderFontSize, "OUTLINE");
    Config.panel.miscHeader:SetPoint( "TOPLEFT", 60, -30);---50 );
    Config.panel.miscHeader:SetText("|cffffff00 - Misc - ");

    -- Surrender to Madness Sound Checkbox
    Config.panel.surrenderSound = CreateFrame("CheckButton", "SurrenderSoundCheckBox", Config.panel, "UICheckButtonTemplate");
    Config.panel.surrenderSound:ClearAllPoints();
    Config.panel.surrenderSound:SetPoint("TOPLEFT", 15, -50);
    _G[Config.panel.surrenderSound:GetName().."Text"]:SetFont(Font, CheckBoxFontSize, "OUTLINE");
    _G[Config.panel.surrenderSound:GetName().."Text"]:SetText("|cffFFFFFF Surrender to Madness Sound Effect");

    -- Surrender to Madness Text Checkbox
    Config.panel.surrenderText = CreateFrame("CheckButton", "SurrenderTextCheckBox", Config.panel, "UICheckButtonTemplate");
    Config.panel.surrenderText:ClearAllPoints();
    Config.panel.surrenderText:SetPoint("TOPLEFT", 15, -75);
    _G[Config.panel.surrenderText:GetName().."Text"]:SetFont(Font, CheckBoxFontSize, "OUTLINE");
    _G[Config.panel.surrenderText:GetName().."Text"]:SetText("|cffFFFFFF Surrender to Madness Text Warning");

    -- Surrender to Madness Text Checkbox
    Config.panel.bennyHill = CreateFrame("CheckButton", "BennyHillCheckBox", Config.panel, "UICheckButtonTemplate");
    Config.panel.bennyHill:ClearAllPoints();
    Config.panel.bennyHill:SetPoint("TOPLEFT", 15, -100);
    _G[Config.panel.bennyHill:GetName().."Text"]:SetFont(Font, CheckBoxFontSize, "OUTLINE");
    _G[Config.panel.bennyHill:GetName().."Text"]:SetText("|cffFFFFFF Benny Will Wipe Announce");

    -- Surrender to Madness Text Checkbox
    Config.panel.bloodLust = CreateFrame("CheckButton", "BloodlustCheckBox", Config.panel, "UICheckButtonTemplate");
    Config.panel.bloodLust:ClearAllPoints();
    Config.panel.bloodLust:SetPoint("TOPLEFT", 15, -125);
    _G[Config.panel.bloodLust:GetName().."Text"]:SetFont(Font, CheckBoxFontSize, "OUTLINE");
    _G[Config.panel.bloodLust:GetName().."Text"]:SetText("|cffFFFFFF Bloodlust / Timewarp Announce");
end

-- Initialise the Config System
local function Init()
    BuildWindow_Primary();

    -- Init Finished
    print("|cffffff00LHPToolkit " .. GetAddOnMetadata("LHPToolkit", "Version") .. " Initialised - '/lhp config' for configuration");
end

-- End of File, Call Init and then add the config to the Blizzard Interface Options window
Init();
InterfaceOptions_AddCategory(Config.panel);

-- Credit : BlizzBugsSuck (Shefki, Phanx) - http://www.wowinterface.com/downloads/info17002-BlizzBugsSuck.html
-- Fix InterfaceOptionsFrame_OpenToCategory not actually opening the category (and not even scrolling to it) Used by the MicroMenu
-- Confirmed still broken in 7.0.3.21973 (7.0.3)
do
	local function get_panel_name(panel)
		local tp = type(panel)
		local cat = INTERFACEOPTIONS_ADDONCATEGORIES
		if tp == "string" then
			for i = 1, #cat do
				local p = cat[i]
				if p.name == panel then
					if p.parent then
						return get_panel_name(p.parent)
					else
						return panel
					end
				end
			end
		elseif tp == "table" then
			for i = 1, #cat do
				local p = cat[i]
				if p == panel then
					if p.parent then
						return get_panel_name(p.parent)
					else
						return panel.name
					end
				end
			end
		end
	end

	local function InterfaceOptionsFrame_OpenToCategory_Fix(panel)
		if doNotRun or InCombatLockdown() then return end
		local panelName = get_panel_name(panel)
		if not panelName then return end -- if its not part of our list return early
		local noncollapsedHeaders = {}
		local shownpanels = 0
		local mypanel
		local t = {}
		local cat = INTERFACEOPTIONS_ADDONCATEGORIES
		for i = 1, #cat do
			local panel = cat[i]
			if not panel.parent or noncollapsedHeaders[panel.parent] then
				if panel.name == panelName then
					panel.collapsed = true
					t.element = panel
					InterfaceOptionsListButton_ToggleSubCategories(t)
					noncollapsedHeaders[panel.name] = true
					mypanel = shownpanels + 1
				end
				if not panel.collapsed then
					noncollapsedHeaders[panel.name] = true
				end
				shownpanels = shownpanels + 1
			end
		end
		local Smin, Smax = InterfaceOptionsFrameAddOnsListScrollBar:GetMinMaxValues()
		if shownpanels > 15 and Smin < Smax then
			local val = (Smax/(shownpanels-15))*(mypanel-2)
			InterfaceOptionsFrameAddOnsListScrollBar:SetValue(val)
		end
		doNotRun = true
		InterfaceOptionsFrame_OpenToCategory(panel)
		doNotRun = false
	end

	hooksecurefunc("InterfaceOptionsFrame_OpenToCategory", InterfaceOptionsFrame_OpenToCategory_Fix)
end
